class DocsController < ApplicationController

  before_filter :find_project, :only => [:index, :add, :copy, :update_documents]
  before_filter :find_doc, :only => [:show, :show_traceability, :export_html, :edit, :destroy, :toc, :content, :derive, :update_doc_element_values]
  before_filter :authorize

  include DocElementsHelper
  
  def index
    @docs = Doc.find(:all, :conditions => "project_id = #{@project.id}").sort
  end

  def add
    @doc = Doc.new(params[:doc])
    if request.post? 
      @doc.project_id = @project.id
      @doc.user_id = User.current.id
      if @doc.save
        flash[:notice] = l(:notice_successful_create)
        redirect_to :action => :index, :id => @project.id
      end
    end
  end

  def copy
    if request.post?
      traceability_docs = []
      traceabilities = []
      if params[:documents_select] != nil
        params[:documents_select].each do |doc_id|
          copy_doc(:project => @project, 
                   :doc => Doc.find(doc_id), 
                   :doc_ids => params[:documents_select], 
                   :traceability_docs => traceability_docs, 
                   :traceabilities => traceabilities)
        end
        if params[:copy_also_traceability] == "on"
          traceability_docs.each do |traceability_doc|
            new_traceability_doc = TraceabilityDoc.new
            new_traceability_doc.traced_id = traceability_doc[:new_traced_id]
            new_traceability_doc.tracer_id = traceability_doc[:new_tracer_id]
            new_traceability_doc.project_id = @project.id
            new_traceability_doc.user_id = User.current.id
            new_traceability_doc.save
            traceability_doc[:traceability_doc_id] = new_traceability_doc.id
          end
          traceabilities.each do |traceability|
            tracer = DocElement.find(traceability[:tracer_id])
            if tracer != nil
              row = search_traceability(:traceabilities => traceability_docs, 
                                        :element => tracer.doc, 
                                        :is_tracer => false)
              if row != nil
                new_traceability = Traceability.new
                new_traceability.traceability_doc_id = traceability_docs[row][:traceability_doc_id]
                new_traceability.traced_id = traceability[:new_traced_id]
                new_traceability.tracer_id = traceability[:new_tracer_id]
                new_traceability.save
              end
            end
          end
        end
      end
      redirect_to :action => :index, :id => @project.id
    else
      @projects = []
      Project.find(:all).each do |project|
        docs = Doc.find(:all, :conditions => "project_id = #{project.id}")
        @projects << {:project => project, :docs => docs, :selected => (project.id == @project.id)}
      end
      @root_projects = Project.find(:all, :conditions => "parent_id IS NULL AND status = 1")
      @project_id = params[:project_id] || @project.id
    end
  end
  
  def update_documents
    @project_id = params[:project_id]
    @documents = Doc.find(:all, :conditions => "project_id = #{@project_id}")
    render :layout => false
  end
  
  def show
    @root_doc_elements = DocElement.find(:all, :conditions => "doc_id = #{@doc.id} AND parent_id IS NULL")
  end
  
  def show_traceability
    @show_traced = params[:show_traced] == "1"
    @export = params[:export] == "1"
    @root_doc_elements = DocElement.find(:all, :conditions => "doc_id = #{@doc.id} AND parent_id IS NULL")
    @selected_element_type = params[:selected_element_type].to_i
    @values = []
    @selected_doc_element_values = params[:selected_doc_element_values]
    @selected_doc_element_values.delete("") unless @selected_doc_element_values.nil? 
    @docs = []
    @docs << @doc
    @traceability_docs = []
    get_traceability_docs(:doc => @doc, 
                          :show_traced => @show_traced,
                          :docs => @docs, 
                          :traceability_docs => @traceability_docs)
    @doc_elements = []
    get_doc_elements(:doc => @doc,
                     :parent_doc_element => nil,
                     :doc_elements => @doc_elements,
                     :selected_element_type => @selected_element_type,
                     :selected_doc_element_values => @selected_doc_element_values)
    if @export
      send_data(render(:layout => false), :type => "text/html", :filename => "#{l(:traceability_docs_title)} - #{@doc.name}.html")
    end
  end
  
  def update_doc_element_values
    selected_element_type = params[:selected_element_type].to_i
    @doc_element_values = {}
    @doc.doc_elements.each do |doc_element|
      if !doc_element.is_section?
        doc_element.doc_element_values.each do |doc_element_value|
          if doc_element_value.element_field_id == selected_element_type
            value = show_value(doc_element_value)
            if doc_element_value.value != nil and
               !(@doc_element_values.include?(doc_element_value.value))
              @doc_element_values[doc_element_value.value] = value
            end
          end
        end
      end
    end
    @doc_element_values[""] = ""
    @selected_doc_element_values = params[:selected_doc_element_values]
    render :layout => false
  end
    
  def export_html
    @root_doc_elements = DocElement.find(:all, :conditions => "doc_id = #{@doc.id} AND parent_id IS NULL")
    send_data(render(:layout => false), :type => "text/html", :filename => "#{@doc.name}.html")
  end

  def edit
    if request.post? and @doc.update_attributes(params[:doc])
      flash[:notice] = l(:notice_successful_update)
      redirect_to :action => :index, :id => @doc.project.id
    else
      @root_doc_elements = DocElement.find(:all, :conditions => "doc_id = #{@doc.id} AND parent_id IS NULL")
    end
  end
  
  def destroy
    project_id = @doc.project.id
    @doc.destroy
    flash[:notice] = l(:notice_successful_delete)
    redirect_to :action => :index, :id => project_id
  rescue
    flash[:error] = l(:notice_unable_delete)
    redirect_to :action => :index, :id => project_id
  end

  def derive
    @selected_element_type = params[:selected_element_type].to_i
    @values = []
    @selected_doc_element_values = params[:selected_doc_element_values]
    @selected_doc_element_values.delete("") unless @selected_doc_element_values.nil? 
    if request.post?
      @doc_to_derive = @doc
      @doc = Doc.new(params[:doc])
      @doc.project_id = @project.id
      @doc.user_id = User.current.id
      if @doc.save
        copy_sections_without_children = params[:copy_sections_without_children] == "1"
        copy_element_content = params[:copy_element_content] == "1"
        trace_derived_doc = params[:trace_derived_doc] == "1"
          
        if trace_derived_doc
          new_traceability_doc = TraceabilityDoc.new
          new_traceability_doc.traced_id = @doc_to_derive.id
          new_traceability_doc.tracer_id = @doc.id
          new_traceability_doc.project_id = @project.id
          new_traceability_doc.user_id = User.current.id
          new_traceability_doc.save
        end
          
        DocElement.find(:all, :conditions => "doc_id = #{@doc_to_derive.id} AND parent_id IS NULL").each do |doc_element|
          copy_doc_element(:doc => @doc_to_derive, 
                           :doc_element => doc_element, 
                           :new_doc => @doc,
                           :copy_doc_element_values => false,
                           :copy_sections_without_children => copy_sections_without_children,
                           :copy_element_content => copy_element_content,
                           :trace_doc => trace_derived_doc,
                           :traceability_doc => new_traceability_doc,
                           :selected_element_type => @selected_element_type,
                           :selected_doc_element_values => @selected_doc_element_values)
        end
          
        flash[:notice] = l(:notice_successful_create)
        redirect_to :action => :index, :id => @project.id
      end
    else
      @doc_to_derive = @doc
      @doc = Doc.new
      @doc.name = copy_doc_name(:project => @project, 
                                :name => @doc_to_derive.name, 
                                :label => :label_derive_index_of_name)
    end
  end
      
private

  def find_project
    @project = Project.find(params[:id])
  rescue ActiveRecord::RecordNotFound
    render_404
  end
  
  def find_doc
    @doc = Doc.find(params[:id])
    @project = @doc.project
  rescue ActiveRecord::RecordNotFound
    render_404
  end

  # Parameters: project, doc, doc_ids, traceability_docs, traceabilities
  # copy_doc_element_values = true, copy_sections_without_children = true,
  # copy_element_content = true, trace_doc = false, traceability_doc
  def copy_doc(params)
    params[:copy_doc_element_values] = true unless params[:copy_doc_element_values] != nil
    params[:copy_sections_without_children] = true unless params[:copy_sections_without_children] != nil
    params[:copy_element_content] = true unless params[:copy_element_content] != nil
    params[:trace_doc] = false unless params[:trace_doc] != nil

    new_doc = Doc.new
    new_doc.project_id = params[:project].id
    new_doc.user_id = User.current.id
    new_doc.element_type_id = params[:doc].element_type_id
    new_doc.name = copy_doc_name(:project => params[:project], 
                                 :name => params[:doc].name)
    new_doc.description = params[:doc].description
    new_doc.version = params[:doc].version
    if new_doc.save
      ok = true
      if params[:doc_ids] != nil and params[:traceability_docs] != nil
        params[:doc].tracers.each do |tracer|
          update_copy_doc_traceability_docs(:doc_ids => params[:doc_ids], 
                                            :traceability_docs => params[:traceability_docs], 
                                            :doc => params[:doc], 
                                            :new_doc => new_doc, 
                                            :element => tracer, 
                                            :is_tracer => false)
        end
        params[:doc].traceds.each do |traced|
          update_copy_doc_traceability_docs(:doc_ids => params[:doc_ids], 
                                            :traceability_docs => params[:traceability_docs], 
                                            :doc => params[:doc], 
                                            :new_doc => new_doc, 
                                            :element => traced, 
                                            :is_tracer => true)
        end
      end
      DocElement.find(:all, :conditions => "doc_id = #{params[:doc].id} AND parent_id IS NULL").each do |doc_element|
        ok = false unless copy_doc_element(:doc => params[:doc], 
                                           :doc_element => doc_element, 
                                           :new_doc => new_doc, 
                                           :doc_ids => params[:doc_ids], 
                                           :traceability_docs => params[:traceability_docs], 
                                           :traceabilities => params[:traceabilities],
                                           :copy_doc_element_values => params[:copy_doc_element_values],
                                           :copy_sections_without_children => params[:copy_sections_without_children],
                                           :copy_element_content => params[:copy_element_content],
                                           :trace_doc => params[:trace_doc],
                                           :traceability_doc => params[:traceability_doc])
      end
      new_doc.destroy unless ok
    end
  end

  # Parameters: doc, doc_element, new_doc, doc_ids, traceability_docs, traceabilities, 
  # new_parent_id, copy_doc_element_values = true, copy_sections_without_children = true,
  # copy_element_content = true, trace_doc = false, traceability_doc,
  # selected_element_type = nil, selected_doc_element_values = nil
  def copy_doc_element(params)
    params[:copy_doc_element_values] = true unless params[:copy_doc_element_values] != nil
    params[:copy_sections_without_children] = true unless params[:copy_sections_without_children] != nil
    params[:copy_element_content] = true unless params[:copy_element_content] != nil
    params[:trace_doc] = false unless params[:trace_doc] != nil

    ok = true
    copy_this_doc_element = true
    if params[:doc_element].is_section?
      if !params[:copy_sections_without_children] and params[:doc_element].has_only_sections?
        copy_this_doc_element = false
      end
    else
      if !params[:selected_element_type].nil? and params[:selected_element_type] != 0 and
         !params[:selected_doc_element_values].nil? and params[:selected_doc_element_values].length > 0
        copy_this_doc_element = false
        params[:doc_element].doc_element_values.each do |doc_element_value|
          if doc_element_value.element_field_id == params[:selected_element_type] and
             params[:selected_doc_element_values].include?(doc_element_value.value)
            copy_this_doc_element = true
          end
        end
      end
    end 
    if copy_this_doc_element
      new_doc_element = DocElement.new
      new_doc_element.doc_id = params[:new_doc].id
      new_doc_element.parent_id = params[:new_parent_id]
      new_doc_element.user_id = User.current.id
      new_doc_element.name = params[:doc_element].name
      if params[:copy_element_content]
        new_doc_element.content = params[:doc_element].content
      end
      new_doc_element.is_section = params[:doc_element].is_section
      new_doc_element.position = params[:doc_element].position
      if new_doc_element.save
        if params[:copy_doc_element_values]
          DocElementValue.find(:all, :conditions => "doc_element_id = #{params[:doc_element].id}").each do |doc_element_value|
            new_doc_element_value = DocElementValue.new
            new_doc_element_value.doc_element_id = new_doc_element.id
            new_doc_element_value.element_field_id = doc_element_value.element_field_id
            new_doc_element_value.value = doc_element_value.value
            ok = false unless new_doc_element_value.save
          end
        end
        if params[:copy_element_content]
          ok = false unless copy_attachments(:doc_element => params[:doc_element],
                                           :new_doc_element => new_doc_element)
        end
        if params[:doc_ids] != nil and params[:traceabilities] != nil
          params[:doc_element].tracers.each do |tracer|
            update_copy_doc_traceabilities(:doc_ids => params[:doc_ids], 
                                           :traceabilities => params[:traceabilities], 
                                           :doc_element => params[:doc_element], 
                                           :element => tracer, 
                                           :new_doc_element => new_doc_element, 
                                           :is_tracer => false)
          end
          params[:doc_element].traceds.each do |traced|
            update_copy_doc_traceabilities(:doc_ids => params[:doc_ids], 
                                           :traceabilities => params[:traceabilities], 
                                           :doc_element => params[:doc_element], 
                                           :element => traced, 
                                           :new_doc_element => new_doc_element, 
                                           :is_tracer => true)
          end
        end
        if params[:trace_doc] and params[:traceability_doc] != nil
          new_traceability = Traceability.new
          new_traceability.traceability_doc_id = params[:traceability_doc].id
          new_traceability.traced_id = params[:doc_element].id
          new_traceability.tracer_id = new_doc_element.id
          new_traceability.save
        end
        DocElement.find(:all, :conditions => "doc_id = #{params[:doc].id} AND parent_id = #{params[:doc_element].id}").each do |child_doc_element|
          ok = false unless copy_doc_element(:doc => params[:doc], 
                                             :doc_element => child_doc_element, 
                                             :new_doc => params[:new_doc], 
                                             :doc_ids => params[:doc_ids], 
                                             :traceability_docs => params[:traceability_docs], 
                                             :traceabilities => params[:traceabilities], 
                                             :new_parent_id => new_doc_element.id,
                                             :copy_doc_element_values => params[:copy_doc_element_values],
                                             :copy_sections_without_children => params[:copy_sections_without_children],
                                             :copy_element_content => params[:copy_element_content],
                                             :trace_doc => params[:trace_doc],
                                             :traceability_doc => params[:traceability_doc],
                                             :selected_element_type => params[:selected_element_type],
                                             :selected_doc_element_values => params[:selected_doc_element_values])
        end
      else
        ok = false
      end
    end
    ok
  end

  # Parameters: doc_element, new_doc_element
  def copy_attachments(params)
    ok = true
    params[:doc_element].attachments.each do |attachment|
      new_attachment = attachment.clone
      new_attachment.container_id = params[:new_doc_element].id
      new_attachment.downloads = 0
      new_attachment.author_id = User.current.id
      new_attachment.disk_filename = generate_disk_filename(:filename => attachment.filename)
      FileUtils.cp(attachment.diskfile, new_attachment.diskfile)
      ok = false unless new_attachment.save
    end
    ok
  end

  # Parameters: filename
  def generate_disk_filename(params)
    disk_filename = DateTime.now.strftime("%y%m%d%H%M%S") + "_"
    if params[:filename] =~ %r{^[a-zA-Z0-9_\.\-]*$}
      disk_filename << params[:filename]
    else
      disk_filename << Digest::MD5.hexdigest(params[:filename])
      disk_filename << $1 if params[:filename] =~ %r{(\.[a-zA-Z0-9]+)$}
    end
    disk_filename
  end
    
  # Parameters: project, name, index = 1, label = :label_copy_index_of_name
  def copy_doc_name(params)
    params[:index] = 1 unless params[:index] != nil
    params[:label] = :label_copy_index_of_name unless params[:label] != nil
      
    copy_name = l(params[:label], params[:index], params[:name])
    if Doc.find(:all, :conditions => "project_id = #{params[:project].id} AND name = \"#{copy_name}\"").length > 0
      copy_doc_name(:project => params[:project], 
                    :name => params[:name], 
                    :index => params[:index] + 1,
                    :label => params[:label])
    else
      copy_name
    end
  end

  # Parameters: doc_ids, traceabilities, doc_element, element, new_doc_element, is_tracer
  def update_copy_doc_traceabilities(params)
    row = search_traceability(:traceabilities => params[:traceabilities], 
                              :element => params[:element], 
                              :is_tracer => params[:is_tracer])
    if row == nil
      included = params[:doc_ids].include?(params[:element].doc_id.to_s)
      if params[:is_tracer]
        params[:traceabilities] << {:tracer_id => params[:doc_element].id,
                                    :new_tracer_id => params[:new_doc_element].id,
                                    :traced_id => params[:element].id,
                                    :new_traced_id => included ? nil : params[:element].id}
      else
        params[:traceabilities] << {:tracer_id => params[:element].id,
                                    :new_tracer_id => included ? nil : params[:element].id,
                                    :traced_id => params[:doc_element].id,
                                    :new_traced_id => params[:new_doc_element].id}
      end
    else
      if params[:is_tracer]
        params[:traceabilities][row][:new_tracer_id] = params[:new_doc_element].id
      else
        params[:traceabilities][row][:new_traced_id] = params[:new_doc_element].id
      end
    end
  end
  
  # Parameters: doc_ids, traceability_docs, doc, new_doc, element, is_tracer
  def update_copy_doc_traceability_docs(params)
    row = search_traceability(:traceabilities => params[:traceability_docs], 
                              :element => params[:element], 
                              :is_tracer => params[:is_tracer])
    if row == nil
      included = params[:doc_ids].include?(params[:element].id.to_s)
      if params[:is_tracer]
        params[:traceability_docs] << {:tracer_id => params[:doc].id,
                                       :new_tracer_id => params[:new_doc].id,
                                       :traced_id => params[:element].id,
                                       :new_traced_id => included ? nil : params[:element].id,
                                       :traceability_doc_id => nil}
      else
        params[:traceability_docs] << {:tracer_id => params[:element].id,
                                       :new_tracer_id => included ? nil : params[:element].id,
                                       :traced_id => params[:doc].id,
                                       :new_traced_id => params[:new_doc].id,
                                       :traceability_doc_id => nil}
      end
    else
      if params[:is_tracer]
        params[:traceability_docs][row][:new_tracer_id] = params[:new_doc].id
      else
        params[:traceability_docs][row][:new_traced_id] = params[:new_doc].id
      end
    end
  end
  
  # Parameters: traceabilities, element, is_tracer
  def search_traceability(params)
    row_index = nil
    params[:traceabilities].each_index do |i|
      row = params[:traceabilities][i]
      if params[:is_tracer]
        if row[:traced_id] == params[:element].id
          row_index = i
          break
        end
      else
        if row[:tracer_id] == params[:element].id
          row_index = i
          break
        end
      end
    end
    row_index
  end

  # Parameters: doc, show_traced, docs, traceability_docs
  def get_traceability_docs(params)
    traceability_docs = TraceabilityDoc.find(:all, :conditions => "#{params[:show_traced] ? "traced" : "tracer"}_id = #{params[:doc].id}")
    traceability_docs.each do |traceability_doc|
      doc = params[:show_traced] ? traceability_doc.tracer : traceability_doc.traced
      if !params[:docs].include?(doc)
        params[:docs] << doc
        params[:traceability_docs] << traceability_doc
        get_traceability_docs(:doc => doc,
                              :show_traced => params[:show_traced],
                              :docs => params[:docs],
                              :traceability_docs => params[:traceability_docs])
      end
    end
  end

  # Parameters: doc, parent_doc_element, doc_elements, selected_element_type, 
  # selected_doc_element_values, level = 0
  def get_doc_elements(params)
    params[:level] = 0 if params[:level].nil?

    DocElement.find(:all, :conditions => "doc_id = #{params[:doc].id} AND parent_id #{params[:parent_doc_element].nil? ? "IS NULL" : " = #{params[:parent_doc_element].id}"}").each do |doc_element|
      print_doc_element = !doc_element.has_only_sections?
      if print_doc_element and !doc_element.is_section and 
         !params[:selected_element_type].nil? and params[:selected_element_type] != 0 and
         !params[:selected_doc_element_values].nil? and params[:selected_doc_element_values].length > 0
        print_doc_element = false
        doc_element.doc_element_values.each do |doc_element_value|
          if doc_element_value.element_field_id == params[:selected_element_type] and
             params[:selected_doc_element_values].include?(doc_element_value.value)
            print_doc_element = true
          end
        end
      end
      if print_doc_element
        params[:doc_elements] << {:doc_element => doc_element,
                                  :level => params[:level]}
        get_doc_elements(:doc => params[:doc],
                         :parent_doc_element => doc_element,
                         :doc_elements => params[:doc_elements],
                         :selected_element_type => params[:selected_element_type],
                         :selected_doc_element_values => params[:selected_doc_element_values],
                         :level => params[:level] + 1)
      end
    end
  end

end
