class ElementTypesController < ApplicationController

  before_filter :require_admin
  before_filter :find_element_type, :except => [:add, :move_element_field, :destroy_element_field]

  def add
    @element_type = ElementType.new(params[:element_type])
    if request.post? and @element_type.save
      flash[:notice] = l(:notice_successful_create)
      redirect_to :controller => :settings, :action => :plugin, :id => "docs"
    end
  end

  def edit
    if request.post? and @element_type.update_attributes(params[:element_type])
      flash[:notice] = l(:notice_successful_update)
      redirect_to :controller => :settings, :action => :plugin, :id => "docs"
    end
  end

  def destroy
    @element_type.destroy
    flash[:notice] = l(:notice_successful_delete)
    redirect_to :controller => :settings, :action => :plugin, :id => "docs"
  rescue
    flash[:error] = l(:notice_unable_delete)
    redirect_to :controller => :settings, :action => :plugin, :id => "docs"
  end
  
  def edit_element_fields
    @element_fields = []
    ElementField.find(:all).sort.each do |element_field|
      if !(@element_type.element_fields.include?(element_field))
        @element_fields << element_field
      end
    end
    @element_fields = @element_fields.collect{|element_field| [element_field.name, element_field.id]}
  end

  def add_element_field
    if request.post?
      element_field_id = params[:element_type][:element_fields].to_i
      if element_field_id != nil && element_field_id > 0
        element_type_element_field = ElementTypeElementField.new
        element_type_element_field.element_type_id = @element_type.id
        element_type_element_field.element_field_id = element_field_id
        if element_type_element_field.save
          flash[:notice] = l(:notice_successful_create)
          redirect_to :action => :edit_element_fields, :id => @element_type.id
          return
        end
      end
      flash[:error] = l(:notice_unable_create)
      redirect_to :action => :edit_element_fields, :id => @element_type.id
    end
  end

  def destroy_element_field
    element_type_id = nil
    element_type_element_field = ElementTypeElementField.find(params[:id])
    element_type_id = element_type_element_field.element_type_id
    element_type_element_field.destroy
    flash[:notice] = l(:notice_successful_delete)
    redirect_to :action => :edit_element_fields, :id => element_type_id
  rescue
    flash[:error] = l(:notice_unable_delete)
    if element_type_id == nil
      redirect_to :controller => :settings, :action => :plugin, :id => "docs"
    else
      redirect_to :action => :edit_element_fields, :id => element_type_id
    end
  end
    
  def move_element_field
    @element_type_element_field = ElementTypeElementField.find(params[:id])
    case params[:position]
    when "highest"
      @element_type_element_field.move_to_top
    when "higher"
      @element_type_element_field.move_higher
    when "lower"
      @element_type_element_field.move_lower
    when "lowest"
      @element_type_element_field.move_to_bottom
    end if params[:position]
    redirect_to :action => :edit_element_fields, :id => @element_type_element_field.element_type_id
  end
  
private
  
  def find_element_type
    @element_type = ElementType.find(params[:id])
  rescue ActiveRecord::RecordNotFound
    render_404
  end

end
