module DocElementsHelper

  include CustomFieldsHelper
  
  def doc_element_value_tag(doc_element_value)	
    element_field = doc_element_value.element_field
    field_name = "values[#{element_field.id}]"
    field_id = "values_#{element_field.id}"
    
    if doc_element_value.value == nil
      value = doc_element_value.element_field.default_value
    else
      value = doc_element_value.value
    end
    
    case element_field.field_format
    when "date"
      text_field_tag(field_name, 
                     value, 
                     :id => field_id, 
                     :size => 10) + 
      image_tag("calendar.png",
                {:id => "#{field_id}_trigger", :class => "calendar-trigger"})
    when "text"
      text_area_tag(field_name, 
                    value, 
                    :id => field_id, 
                    :rows => 8,
                    :style => "width: 97%;")
    when "bool"
      check_box_tag(field_name, 
                    "1", 
                    value == "1", 
                    :id => field_id) + 
      hidden_field_tag(field_name, "0")
    when "list"
      blank_option = element_field.is_required? ?
                     (element_field.default_value.blank? ? "<option value=\"\">--- #{l(:actionview_instancetag_blank_option)} ---</option>" : "") : 
                     "<option></option>"
      select_tag(field_name, 
                 blank_option + options_for_select(element_field.possible_values, value), 
                 :id => field_id)
    else
      text_field_tag(field_name, 
                     value, 
                     :id => field_id)
    end
  end
  
  def doc_element_value_label_tag(doc_element_value)
    content_tag("label", 
                doc_element_value.element_field.name + (doc_element_value.element_field.is_required? ? " <span class=\"required\">*</span>" : ""),
                :for => "values_#{doc_element_value.element_field.id}",
                :class => (doc_element_value.errors.empty? ? nil : "error" ))
  end
  
  def doc_element_value_with_label_tag(doc_element_value)
    doc_element_value_label_tag(doc_element_value) + doc_element_value_tag(doc_element_value)
  end

  def show_value(doc_element_value)
    return "" unless doc_element_value
    format_value(doc_element_value.value, doc_element_value.element_field.field_format)
  end
  
  def format_doc_element_value(doc_element_value, attachments)
    return "" unless doc_element_value.value && !doc_element_value.value.empty?
    case doc_element_value.element_field.field_format
    when "date"
      begin; value = format_date(doc_element_value.value.to_date); rescue; value = doc_element_value.value end
    when "bool"
      value = l_YesNo(doc_element_value.value == "1")
    else
      value = doc_element_value.value
    end
    textilizable("*#{doc_element_value.element_field.name}:* #{value}", :attachments => attachments, :only_path => false)
  end

  def render_header(doc_element)
    header = ""
    if doc_element != nil
      if doc_element.parent != nil 
        header << "#{render_header(doc_element.parent)}."
      end
      header <<= "#{doc_element_index(doc_element)}"
    end
    header
  end
  
  def doc_element_index(doc_element)
    brothers = DocElement.find(:all, :conditions => "doc_id = #{doc_element.doc_id} AND parent_id #{doc_element.parent_id == nil ? "IS NULL" : "= #{doc_element.parent_id}"}")
    index = 0
    i = 1
    brothers.each do |brother|
      if index == 0 and brother.id == doc_element.id
        index = i
      end
      i += 1
    end
    index
  end
  
  # Parameters: doc, doc_element, show = true
  def render_doc_element_values(params)
    params[:show] = true if params[:show].nil?

    html = ""
    if params[:doc] != nil and
       params[:doc_element] != nil and (!params[:doc_element].is_section or !params[:show])
      params[:doc].element_type.element_fields.each do |element_field|
        doc_element_value = params[:doc_element].doc_element_values.first(:conditions => "element_field_id = #{element_field.id}")
        doc_element_value = nil
        params[:doc_element].doc_element_values.each do |value|
          if value.element_field_id == element_field.id
            doc_element_value = value
          end
        end
        if doc_element_value != nil
          if params[:show]
            html << format_doc_element_value(doc_element_value, params[:doc_element].attachments)
          else
            label = doc_element_value_label_tag(doc_element_value)
            value = doc_element_value_tag(doc_element_value)
            if doc_element_value.element_field.field_format == "text"
              html << "<p>#{label}</p>\n#{value}\n"
            else
              html << "<p>#{label}#{value}</p>\n"
            end
          end
        end
      end
    end
    html
  end

  def render_traceability_links(doc_elements)
    links = []
    if !doc_elements.empty?
      doc_elements.each do |doc_element|
        link = ""
        link << link_to_if_authorized(h(doc_element.name), {:controller => :docs, :action => :show, :id => doc_element.doc.id, :anchor => "doc_element_#{doc_element.id}"}, :title => "\"#{doc_element.name}\" #{l(:label_on)} \"#{doc_element.doc.name}\"")
        links << link
      end
    end
    links.join(", ")
  end

  # Parameters: doc, doc_element, level = 0, export = false, row_span = 1
  def render_doc_element_header(params)
    params[:level] = 0 if params[:level].nil?
    params[:export] = false if params[:export].nil?
    params[:row_span] = 1 if params[:row_span].nil?
    
    html = ""
    html << "        <td width=\"1%\" rowspan=\"#{params[:row_span]}\">#{render_header(params[:doc_element])}</td>\n"
    html << "        <td rowspan=\"#{params[:row_span]}\"#{" style=\"padding-left: #{params[:level] * 20}px;\"" unless params[:export]}>\n"
    if !params[:export]
      if params[:doc_element].is_section
        html << "          #{image_tag("folder.png")}\n"
      else
        html << "          #{DocsHelper.plugin_image_tag("element_type_#{params[:doc_element].doc.element_type.icon}.png")}\n"
      end
    end
    if params[:doc_element].is_section or params[:export]
      html << "          #{h(params[:doc_element].name)}\n"
    else
      html << "          #{link_to_if_authorized(h(params[:doc_element].name), {:controller => :docs, :action => :show, :id => params[:doc].id, :anchor => "doc_element_#{params[:doc_element].id}"})}\n"
    end
    html << "        </td>\n"
    html
  end

  # Parameters: doc_element, docs, traceability_docs, show_traced, class, export = false
  def render_doc_element_traceability(params)
    params[:export] = false if params[:export].nil?

    rows = 1
    doc_elements = {}
    doc_elements[params[:doc_element][:doc_element].doc.id] = [params[:doc_element][:doc_element]]
    params[:traceability_docs].each do |traceability_doc|
      if params[:show_traced]
        left_doc = traceability_doc.traced
        tracex_doc = traceability_doc.tracer
      else
        left_doc = traceability_doc.tracer
        tracex_doc = traceability_doc.traced
      end
      tracex_doc_elements = []
      count = 0
      doc_elements[left_doc.id].each do |doc_element|
        traceabilities = Traceability.find(:all, :conditions => "traceability_doc_id = #{traceability_doc.id} AND #{params[:show_traced] ? "traced" : "tracer"}_id = #{doc_element.id}")
        traceabilities.each do |traceability|
          tracex_doc_element = DocElement.find(params[:show_traced] ? traceability.tracer.id : traceability.traced.id)
          if !tracex_doc_elements.include?(tracex_doc_element)
            tracex_doc_elements << tracex_doc_element
            count += 1
          end
        end
      end
      doc_elements[tracex_doc.id] = tracex_doc_elements.sort
      rows = count if count > rows
    end

    html = ""
    html << "<tr#{" class=\"#{params[:class]}\"" if !params[:export]}>\n"
    html << render_doc_element_header(:doc => params[:docs][0], 
                                      :doc_element => params[:doc_element][:doc_element], 
                                      :level => params[:doc_element][:level],
                                      :export => params[:export],
                                      :row_span => rows)

    first = true
    rows.times do |index|
      html << "      <tr#{" class=\"#{params[:class]}\"" if !params[:export]}>\n" unless first
      params[:traceability_docs].each do |traceability_doc|
        tracex_doc = params[:show_traced] ? traceability_doc.tracer : traceability_doc.traced
        if index >= doc_elements[tracex_doc.id].length
          html << "        <td></td>\n"
          html << "        <td></td>\n"
        else
          tracex_doc_element = doc_elements[tracex_doc.id][index]
          html << render_doc_element_header(:doc => tracex_doc, 
                                            :doc_element => tracex_doc_element,
                                            :export => params[:export])
        end
      end
      html << "      </tr>"
      first = false
    end
    
    html
  end

end
