module DocsHelper

  include DocElementsHelper
  
  def tree_select(categories, model, name, selected = nil, level = 0, init = true)
    html = ""
    if init
        html << "<select name=\"#{model}[#{name}]\" id=\"#{model}_#{name}\">\n"
        html << "\t<option value=\"0\""
        html << " selected=\"selected\"" if selected != nil and selected.parent_id == 0
        html << ">Root</option>\n"
    end

    if categories != nil and categories.length > 0
      level += 1
      categories.collect do |cat|
        html << "\t<option value=\"#{cat.id}\" style=\"padding-left:#{level * 10}px\""
        html << ' selected="selected"' if selected != nil and cat.id == selected.parent_id
        html << ">#{cat.name}</option>\n"
        html << tree_select(cat.children, model, name, selected, level, false)
      end
    end
    html << "</select>\n" if init
    return html
  end

  def project_tree_select(projects, current_project_id, selected_id = nil, level = 0, init = true)
    html = ""
    if init
      html << "<select name=\"project_id\" id=\"project_id\" onChange=\"#{remote_function(:update => :documents_select, :url => {:action => :update_documents, :id => current_project_id}, :with => "Form.Element.serialize(this)")};\">\n"
    end
    if projects != nil and projects.length > 0
      projects.collect do |project|
        html << "  <option value=\"#{project.id}\" style=\"padding-left:#{level * 20}px\""
        html << " selected=\"selected\"" if selected_id != nil and project.id == selected_id
        html << ">#{project.name}</option>\n"
        html << project_tree_select(project.children, current_project_id, selected_id, level + 1, false)
      end
    end
    html << "</select>\n" if init
    return html
  end

  # Parameters: doc_elements, editable = true, level = 0, init = true
  def render_toc(params)
    params[:editable] = true if params[:editable].nil?
    params[:level] = 0 if params[:level].nil?
    params[:init] = true if params[:init].nil?
    
    html = ""
    if params[:doc_elements] != nil and params[:doc_elements].length > 0
      if params[:init]
        html << "<table class=\"#{params[:editable] ? "list" : ""}\">\n"
        if params[:editable]
          html << "  <thead>\n"
          html << "    <th width=\"1%\"></th>\n"
          html << "    <th>#{l(:field_name)}</th>\n"
          html << "    <th width=\"25%\">#{l(:button_sort)}</th>\n" if params[:editable]
          html << "    <th width=\"15%\"></th>\n" if params[:editable]
          html << "  </thead>\n"
        end
        html << "  <tbody>\n"
      end
      index = 1
      params[:doc_elements].collect do |doc_element|
        html << "    <tr class=\"#{params[:editable] ? cycle("odd", "even") : ""}\">\n"
        html << "      <td>\n"
        html << "        #{render_header(doc_element)}\n"
        html << "      </td>\n"
        html << "      <td style=\"padding-left: #{params[:level] * 20}px;\">\n"
        if params[:editable]
          javascript = "var toolbar = new jsToolBar($('doc_element_content')); toolbar.draw(); " +
                       "update_doc_element_values_div (); "
          doc_element.doc.element_type.element_fields.each do |element_field|
            if element_field.field_format == "date"
              field_id = "values_#{element_field.id}"
              javascript +=  "Calendar.setup({inputField : '#{field_id}', ifFormat : '%Y-%m-%d', button : '#{field_id}_trigger' }); "
            elsif element_field.field_format == "text"
              javascript += "var toolbar = new jsToolBar($('values_#{element_field.id}')); toolbar.draw(); "
            end
          end
          visible = "document.getElementById('element_doc_div').style.display = 'inline';"
          if doc_element.is_section
            html << "        #{image_tag("folder.png")}\n"
          else
            html << "        #{DocsHelper.plugin_image_tag("element_type_#{doc_element.doc.element_type.icon}.png")}\n"
          end
        end
        if params[:editable]
          html << "        <a name=\"doc_element_toc_#{doc_element.id}\"></a>\n"
          html << "        #{link_to_remote_if_authorized(h(doc_element.name), {:url => {:controller => :doc_elements, :action => :show, :id => doc_element.id}, :update => :element_doc_div}, {:onclick => visible})}\n"
        else
          html << "        #{link_to(h(doc_element.name), :anchor => "doc_element_#{doc_element.id}")}\n"
        end
        html << "      </td>\n"
        if params[:editable]
          html << "      <td align=\"center\">\n"
          html << "        #{link_to_if_authorized(image_tag("arrow_to.png",   :alt => l(:label_sort_up)),       {:controller => :doc_elements, :action => :move, :id => doc_element.id, :position => "up"},      :method => :post, :title => l(:label_sort_up     ))} -\n"
          html << "        #{link_to_if_authorized(image_tag("arrow_from.png", :alt => l(:label_sort_down)),     {:controller => :doc_elements, :action => :move, :id => doc_element.id, :position => "down"},    :method => :post, :title => l(:label_sort_down   ))} -\n"
          html << "        #{link_to_if_authorized(image_tag("2uparrow.png",   :alt => l(:label_sort_highest)),  {:controller => :doc_elements, :action => :move, :id => doc_element.id, :position => "highest"}, :method => :post, :title => l(:label_sort_highest))}\n"
          html << "        #{link_to_if_authorized(image_tag("1uparrow.png",   :alt => l(:label_sort_higher)),   {:controller => :doc_elements, :action => :move, :id => doc_element.id, :position => "higher"},  :method => :post, :title => l(:label_sort_higher ))} -\n"
          html << "        #{link_to_if_authorized(image_tag("1downarrow.png", :alt => l(:label_sort_lower)),    {:controller => :doc_elements, :action => :move, :id => doc_element.id, :position => "lower"},   :method => :post, :title => l(:label_sort_lower  ))}\n"
          html << "        #{link_to_if_authorized(image_tag("2downarrow.png", :alt => l(:label_sort_lowest)),   {:controller => :doc_elements, :action => :move, :id => doc_element.id, :position => "lowest"},  :method => :post, :title => l(:label_sort_lowest ))}\n"
          html << "      </td>\n"
          html << "      <td align=\"center\">\n"
          html << "        #{link_to_remote_if_authorized(image_tag("add.png", :alt => l(:button_add)), {:url => {:controller => :doc_elements, :action => :new, :id => @doc.id, :parent_id => doc_element.id, :position => doc_element.children.length + 1}, :update => :element_doc_div, :complete => javascript}, {:style => doc_element.is_section ? "" : "visibility: hidden;", :onclick => visible})}\n"
          html << "        #{link_to_remote_if_authorized(image_tag("edit.png", :alt => l(:button_edit)), {:url => {:controller => :doc_elements, :action => :edit, :id => doc_element.id}, :update => :element_doc_div, :complete => javascript}, {:onclick => visible})}\n"
          html << "        #{link_to_if_authorized(image_tag("delete.png", :alt => l(:button_delete)), {:controller => :doc_elements, :action => :destroy, :id => doc_element.id}, :confirm => l(:text_are_you_sure), :method => :post)}\n"
          html << "      </td>\n"
        end
        html << "    </tr>\n"
        html << render_toc(:doc_elements => doc_element.children, 
                           :editable => params[:editable], 
                           :level => params[:level] + 1, 
                           :init => false)
        index += 1
      end
      if params[:init]
        html << "  </tbody>\n"
        html << "</table>\n"
        html << "<a name=\"doc_element_toc_end\"></a>\n"
        reset_cycle
      end
    end
    return html
  end
  
  # Parameters: doc, doc_elements, level = 0, export = false
  def render_doc(params)
    params[:level] = 0 if params[:level].nil?
    params[:export] = false if params[:export].nil?
      
    html = ""
    if params[:doc] != nil and params[:doc_elements] != nil and params[:doc_elements].length > 0
      params[:doc_elements].collect do |doc_element|
        header_level = params[:level] > 5 ? 5 : params[:level];
        if !params[:export]
          html << "<div class=\"doc\" style=\"padding-left: #{params[:level] * 30}px;\">"
          html << "  <a name=\"doc_element_#{doc_element.id}\"></a>\n"
        end
        html << "  <h#{header_level + 1} style=\"#{header_style} #{header_styles[header_level]}\">#{"#{render_header(doc_element)} " unless params[:export]}#{h(doc_element.name)}</h#{header_level + 1}>\n"
        html << "  #{textilizable(doc_element.content, 
                                  :attachments => doc_element.attachments,
                                  :only_path => false)}\n"
        html << render_doc_element_values(:doc => params[:doc], 
                                          :doc_element => doc_element, 
                                          :show => true)
        if !params[:export]
          if !doc_element.tracers.empty?
            html << "  <p>\n"
            html << "    <b>#{l(:field_element_tracer_plural)}:</b>\n"
            html << "    #{render_traceability_links(doc_element.tracers)}\n"
            html << "  </p>\n"
          end
          if !doc_element.traceds.empty?
            html << "  <p>\n"
            html << "    <b>#{l(:field_element_traced_plural)}:</b>\n"
            html << "    #{render_traceability_links(doc_element.traceds)}\n"
            html << "  </p>\n"
          end
          html << "</div>\n"
        end
        html << render_doc(:doc => params[:doc], 
                           :doc_elements => doc_element.children, 
                           :level => params[:level] + 1,
                           :export => params[:export])
      end
    end
    return html
  end

private
  
  def header_style
    "border: 0px; margin: 0px; padding: 20px 0px 10px 0px;"
  end
  
  def header_styles
    {
      0 => "font-size: 24px;",
      1 => "font-size: 20px;",
      2 => "font-size: 17px;",
      3 => "font-size: 15px;",
      4 => "font-size: 14px;",
      5 => "font-size: 13px;"
    }
  end
  
end
